///File : DevisManager.cpp
//------------------------------------------------------------------------------
/*
    This file is part of DevisMaker.

    DevisMaker is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DevisMaker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DevisMaker.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "DevisManager.hpp"
//------------------------------------------------------------------------------
DevisManager::DevisManager(const std::string &sectionDBPath, const std::string &itemDBPath, const std::string &cfgPath ) : ItemSectionDBBase(sectionDBPath, itemDBPath )
{
    #if defined USE_MULTIPLE_CONFIG
    m_dvs = NULL;
    m_currentDevisId = 0;
    m_LastDevisId = 0;
    #else
    m_dvs = &m_realDvs;
    m_isOpen = false;
    m_dvs->isDevisModified = false;
    #endif

    m_cfg = &m_realCfg;
    m_cfg->LoadFromFile(cfgPath);
}
//------------------------------------------------------------------------------

bool DevisManager::NewSection(const std::string &sectionID)
{
    StructSection tempSection;
    tempSection.sectionID = sectionID;
    m_dvs->sections.push_back(tempSection);
    m_dvs->isDevisModified = true;

    BackupSaveCurrent(BACKUP_SECTION_ADDED, sectionID);

    return true;
}
//------------------------------------------------------------------------------
std::string DevisManager::GetSectionName(const size_t &posSection) const
{
    return m_dvs->sections[posSection].sectionID;
}
//------------------------------------------------------------------------------
void DevisManager::ThrowIfNotOpenAndOk()
{
    if(!IsOk())
    {
        throw ticpp::Exception("DevisManager is not Ok !");
    }
    else if(!IsOpen())
    {
        throw ticpp::Exception("No devis are Opened !");
    }
}
//------------------------------------------------------------------------------
bool DevisManager::CleanDevis()
{
    ThrowIfNotOpenAndOk();

    // Enleve tous les items non présents dans la DB.
    bool isItemRemoved = false;

    for(int sectionPos = 0; sectionPos < m_dvs->sections.size(); sectionPos++)
    {
        for(int itemPos = m_dvs->sections[sectionPos].items.size() - 1; itemPos >= 0; itemPos--)
        {
            if(! ItemExists(m_dvs->sections[sectionPos].items[itemPos].ref))
            {
                m_dvs->sections[sectionPos].items.erase(m_dvs->sections[sectionPos].items.begin() + itemPos);
                m_dvs->isDevisModified = true;
                isItemRemoved = true;
            }
        }
    }

    if(isItemRemoved)
    {
        BackupStart(BACKUP_NEW);
    }

    return isItemRemoved;
}
//------------------------------------------------------------------------------
void DevisManager::ReloadSectionItemDB(const std::string &sectionDBPath, const std::string &itemDBPath)
{
    InitSectionItemDB(sectionDBPath, itemDBPath ); // O, recharge la DB.
}
//------------------------------------------------------------------------------
ConfigManager *DevisManager::GetDevisCfg()
{
    return &m_dvs->cfg;
}
//------------------------------------------------------------------------------
GeneralCfg *DevisManager::GetGeneralCfg()
{
    return m_cfg;
}

//------------------------------------------------------------------------------
double DevisManager::GetDumpChargeHT() const
{
    if(m_dvs->cfg.GetBool("isDumpCharges"))
    {
        return m_dvs->cfg.GetDouble("dumpCharges");
    }
    else
    {
        return 0;
    }
}
//------------------------------------------------------------------------------
double DevisManager::GetPortHT() const
{
    if(m_dvs->cfg.GetBool("isPort"))
    {
        return m_dvs->cfg.GetDouble("port");
    }
    else
    {
        return 0;
    }
}
//------------------------------------------------------------------------------

double DevisManager::GetTotalBrutHT() const
{
    return GetSectionsPriceHT() + GetDumpChargeHT() + GetPortHT();
}
//------------------------------------------------------------------------------

double DevisManager::GetPriceDiscountRateHT() const
{
    return GetTotalBrutHT() * (m_dvs->cfg.GetDouble("discountRate") / double(100));
}
//------------------------------------------------------------------------------

double DevisManager::GetDiscountHT() const
{
    return m_dvs->cfg.GetDouble("discount");
}
//------------------------------------------------------------------------------

double DevisManager::GetTotalNetHT() const
{
    return GetTotalBrutHT() - GetDiscountHT() - GetPriceDiscountRateHT() + GetEcoPartTotalPrice();
}
//------------------------------------------------------------------------------

double DevisManager::GetTVAPrice() const
{
    double taxe = 0;

    for(int sectionPos = 0; sectionPos < m_dvs->sections.size(); sectionPos++)
    {
        for(int itemPos = 0; itemPos < m_dvs->sections[sectionPos].items.size(); itemPos++)
        {
            std::string itemID = GetItemRef(sectionPos, itemPos);

            // /!\ GetItemPriceWithQuantityEcoPartHT ==> Eco part passe par la TVA !
            taxe += GetItemPriceWithQuantityEcoPartHT(sectionPos, itemPos) * (GetItemTaxe(itemID) / double(100));
        }
    }

    double servicesTVA = m_dvs->cfg.GetDouble("servicesTVA") / 100.f;
    taxe += GetPortHT() * servicesTVA;
    taxe += GetDumpChargeHT() * servicesTVA;

    return taxe;
}
//------------------------------------------------------------------------------
void DevisManager::AddToTVATbl(std::vector<StructTVALine> &tempTbl, const double &currentBase, const double &currentTVA) const
{
    double currentTotal = currentBase * (currentTVA / double(100));

    int tblPos = -1;
    // test si TVA pas encore dans le tableau...
    for(int i = 0; i < tempTbl.size(); i++)
    {
        if(tempTbl[i].TVA == currentTVA)
        {
            tblPos = i;
        }
    }

    // Si TVA existe déjà ==> Ajout
    if(tblPos != -1)
    {
        tempTbl[tblPos].base += currentBase;
        tempTbl[tblPos].total += currentTotal;
    }

    else
    {
        StructTVALine tempLine;
        tempLine.TVA = currentTVA;
        tempLine.base = currentBase;
        tempLine.total = currentTotal;

        tempTbl.push_back(tempLine);
    }
}
//------------------------------------------------------------------------------
std::vector<StructTVALine> DevisManager::GetTVATbl() const
{
    std::vector<StructTVALine> tempTbl;
    double currentBase;
    double currentTVA;

    for(int sectionPos = 0; sectionPos < m_dvs->sections.size(); sectionPos++)
    {
        for(int itemPos = 0; itemPos < m_dvs->sections[sectionPos].items.size(); itemPos++)
        {
            std::string itemID = GetItemRef(sectionPos, itemPos);

            // /!\ GetItemPriceWithQuantityEcoPartHT ==> Eco part passe par la TVA !
            currentBase = GetItemPriceWithQuantityEcoPartHT(sectionPos, itemPos);
            currentTVA = GetItemTaxe(itemID);

            AddToTVATbl(tempTbl, currentBase, currentTVA);
        }
    }

    // Frais de port et de décheterie
    double servicesTVA = m_dvs->cfg.GetDouble("servicesTVA");
    AddToTVATbl(tempTbl, GetDumpChargeHT(), servicesTVA);
    AddToTVATbl(tempTbl, GetPortHT(), servicesTVA);

    // Classement...
    if(tempTbl.size() > 1)
    {
        int lineOk = 0;
        while(lineOk < tempTbl.size() - 1) // Tant que toutes les lignes ne sont pas ok... (-1 car on ne fait que n-1 test (chaque teste pour 2 lignes))
        {
            lineOk = 0;
            for(int i = 0; i < (tempTbl.size() - 1); i++)
            {
                if(tempTbl[i].TVA < tempTbl[i + 1].TVA) // Si pas dans le bonne ordre...
                {
                    // On swap les lignes...
                    StructTVALine tempLine = tempTbl[i + 1];
                    tempTbl[i + 1] = tempTbl[i];
                    tempTbl[i] = tempLine;
                }
                else
                {
                    // Une ligne est ok...
                    lineOk++;
                }
            }
        }
    }

    // Remplissage de la partie "T" :
    for(int i = 0; i < tempTbl.size(); i++)
    {
        tempTbl[i].t = nbToString(i + 1);
    }

    return tempTbl;
}
//------------------------------------------------------------------------------

std::string DevisManager::GetItemTVAid(const int &posSection, const size_t &posItem, const std::vector<StructTVALine> &TVATbl) const
{
    double itemTVA = GetItemTaxe(GetItemRef(posSection, posItem));

    for(int i = 0; i < TVATbl.size(); i++)
    {
        if(TVATbl[i].TVA == itemTVA)
        {
            return TVATbl[i].t;
        }
    }

    return "";
}
//------------------------------------------------------------------------------
double DevisManager::GetEcoPartTotalPrice() const
{
    if(m_dvs->cfg.GetBool("isEcoPart"))
    {
        double totalEcoPart = 0;

        for(int sectionPos = 0; sectionPos < m_dvs->sections.size(); sectionPos++)
        {
            for(int itemPos = 0; itemPos < m_dvs->sections[sectionPos].items.size(); itemPos++)
            {
                totalEcoPart += GetItemEcoPart(GetItemRef(sectionPos, itemPos)) * GetItemQuantity(sectionPos, itemPos);
            }
        }
        return totalEcoPart;
    }
    else
    {
        return 0;
    }

}
//------------------------------------------------------------------------------
double DevisManager::GetTotalTTC() const
{
    return GetTotalNetHT() + GetTVAPrice();
}
//------------------------------------------------------------------------------

double DevisManager::GetPaymentOn() const
{
    return m_dvs->cfg.GetDouble("paymentOn");
}
//------------------------------------------------------------------------------

double DevisManager::GetTotalToPay() const
{
    return GetTotalTTC() - GetPaymentOn();
}
//------------------------------------------------------------------------------
std::vector<std::string> DevisManager::GetSectionsList() const
{
    std::vector<std::string> tempList;

    if(IsOpen())
    {
        for(int i = 0; i < m_dvs->sections.size(); i++)
        {
            tempList.push_back(m_dvs->sections[i].sectionID);
        }
    }
    return tempList;
}
//------------------------------------------------------------------------------
ticpp::Element *DevisManager::PrepareSection(const int posSection) const
{
    ticpp::Element *Section = new ticpp::Element("section");

    Section->InsertEndChild(*PrepareInfo("sectionID", m_dvs->sections[posSection].sectionID));

    ticpp::Element Items("items");
    for(int posItem = 0; posItem < m_dvs->sections[posSection].items.size(); posItem++)
    {
        ticpp::Element Item("item");
        Item.SetAttribute("ref", m_dvs->sections[posSection].items[posItem].ref);
        Item.SetAttribute("quantity", m_dvs->sections[posSection].items[posItem].quantity);
        Items.InsertEndChild(Item);
    }

    Section->InsertEndChild(Items);

    return Section;
}

//------------------------------------------------------------------------------
ticpp::Element *DevisManager::PrepareInfo(const std::string &nom, const std::string &info) const
{
    ticpp::Element *infoElement = new ticpp::Element(nom);
    infoElement->SetText(info);
    return infoElement;
}

//------------------------------------------------------------------------------
void DevisManager::Save(const std::string &filename)
{
    if(filename != "")
    {
        m_dvs->filename = filename;
    }

    ticpp::Document myDoc(m_dvs->filename);

    ticpp::Declaration myDeclaration("1.0","UTF-8","no");

    ticpp::Element Version("version");
    Version.SetText("1");

    ticpp::Element Sections("sections");
    for(int i = 0; i < m_dvs->sections.size(); i++)
    {
        Sections.InsertEndChild(*PrepareSection(i));
    }

    ticpp::Element Base("devismaker_file");
    Base.InsertEndChild(Version);
    ticpp::Element cfgElement = m_dvs->cfg.SaveToElement();
    Base.InsertEndChild(cfgElement);
    Base.InsertEndChild(Sections);

    myDoc.InsertEndChild(myDeclaration);
    myDoc.InsertEndChild(Base);

    myDoc.SaveFile();

    m_dvs->isDevisModified = false;
    m_dvs->backup[m_dvs->backupPos].isDevisModified = false; // Le backup est lui aussi sauvergardé (car c'est le même que les données actueles).
}

//------------------------------------------------------------------------------
void DevisManager::New()
{
    if(IsOk())
    {
        #if defined USE_MULTIPLE_CONFIG
        int newPos = NewDevis();
        SetCurrentDevisId(newPos);
        #else
        Close();
        m_isOpen = true;
        #endif

        m_dvs->filename = "";
        m_dvs->isDevisModified = false; // <== Devis vide = pas besoin d'enregistrer

        m_dvs->cfg = m_cfg->GetDefaultDevisCfg(); // Seul les données sont copiées

        BackupStart(BACKUP_NEW);
    }
}
//------------------------------------------------------------------------------

void DevisManager::Close()
{
    #if defined USE_MULTIPLE_CONFIG
    RemoveDevis(GetCurrentDevisId());
    #else
    m_isOpen = false;
    m_dvs->cfg.ClearAll();
    m_dvs->sections.clear();
    m_dvs->filename = "";
    m_dvs->isDevisModified = false;

    BackupClear();
    #endif


}
//------------------------------------------------------------------------------

int DevisManager::GetSectionItemsNumber(const size_t &posSection) const
{
    if(m_dvs->sections.size() > posSection)
    {
        return m_dvs->sections[posSection].items.size();
    }
    else
    {
        return 0;
    }
}
//------------------------------------------------------------------------------

bool DevisManager::RemoveSection(const size_t &posSection)
{
    if(m_dvs->sections.size() > posSection)
    {
        m_dvs->sections.erase(m_dvs->sections.begin() + posSection);
        m_dvs->isDevisModified = true;

        BackupSaveCurrent(BACKUP_SECTION_REMOVED, GetSectionName(posSection));

        return true;
    }
    else
    {
        throw ticpp::Exception("No section with this pos !");
    }
}
//------------------------------------------------------------------------------
std::string DevisManager::GetDevisSectionName(const size_t &posSection) const
{
    if(m_dvs->sections.size() > posSection)
    {
        return m_dvs->sections[posSection].sectionID;
    }
    else
    {
        throw ticpp::Exception("No section with this pos !");
    }
}
//------------------------------------------------------------------------------
bool DevisManager::RenameSection(const size_t &posSection, const std::string &newName)
{
    if(m_dvs->sections.size() > posSection)
    {
        m_dvs->sections[posSection].sectionID = newName;
        m_dvs->isDevisModified = true;

        BackupSaveCurrent(BACKUP_SECTION_RENAMED, GetSectionName(posSection));

        return true;
    }
    else
    {
        throw ticpp::Exception("No section with this pos !");
    }
}
//------------------------------------------------------------------------------
double DevisManager::GetSectionsPriceHT() const
{
    double price = 0;
    for(size_t i = 0; i < m_dvs->sections.size(); i++)
    {
        price += GetSectionPriceHT(i);
    }
    return price;
}

//------------------------------------------------------------------------------

double DevisManager::GetSectionPriceHT(const size_t &posSection) const
{
    if(m_dvs->sections.size() > posSection)
    {
        double price = 0;
        for(size_t i = 0; i < m_dvs->sections[posSection].items.size(); i++)
        {
            price += GetItemPriceWithQuantityHT(posSection, i);
        }
        return price;
    }
    else
    {
        throw ticpp::Exception("No section with this pos !");
    }
}
//------------------------------------------------------------------------------

bool DevisManager::SwapSections(const size_t &posSection1, const size_t &posSection2)
{
    if(m_dvs->sections.size() > posSection1 && m_dvs->sections.size() > posSection2)
    {
        StructSection tmpSection = m_dvs->sections[posSection1];
        m_dvs->sections[posSection1] = m_dvs->sections[posSection2];
        m_dvs->sections[posSection2] = tmpSection;
        m_dvs->isDevisModified = true;

        BackupSaveCurrent(BACKUP_SECTION_MOVED, GetSectionName(posSection1));

        return true;
    }
    else
    {
        return false;
    }
}
//------------------------------------------------------------------------------

std::vector<std::string> DevisManager::GetItemsList(const size_t &posSection) const
{
    std::vector<std::string> tempList;
    if(m_dvs->sections.size() > posSection)
    {
        for(size_t i = 0; i < m_dvs->sections[posSection].items.size(); i++)
        {
            tempList.push_back(m_dvs->sections[posSection].items[i].ref);
        }
        return tempList;
    }
    else
    {
        throw ticpp::Exception("No section with this pos !");
    }
}

//------------------------------------------------------------------------------
double DevisManager::GetItemQuantity(const int &posSection, const size_t &posItem) const
{
    if(m_dvs->sections.size() > posSection)
    {
        if(m_dvs->sections[posSection].items.size() > posItem)
        {
            return m_dvs->sections[posSection].items[posItem].quantity;
        }
        else
        {
            throw ticpp::Exception("No item with this pos !");
        }
    }
    else
    {
        throw ticpp::Exception("No section with this pos !");
    }
}
//------------------------------------------------------------------------------
double DevisManager::GetItemPriceWithQuantityEcoPartHT(const size_t &posSection, const size_t &posItem) const
{
    if(m_dvs->sections.size() > posSection)
    {
        if(m_dvs->sections[posSection].items.size() > posItem)
        {
            double ecoPart = 0;
            if(m_dvs->cfg.GetBool("isEcoPart"))
            {
                ecoPart = GetItemEcoPart(GetItemRef(posSection, posItem));
            }

            double quantity = GetItemQuantity(posSection, posItem);
            double priceHT = GetItemPriceHT(GetItemRef(posSection, posItem));


            return quantity * (priceHT + ecoPart);
        }
        else
        {
            throw ticpp::Exception("No item with this pos !");
        }
    }
    else
    {
        throw ticpp::Exception("No section with this pos !");
    }
}
//------------------------------------------------------------------------------
double DevisManager::GetItemPriceWithQuantityHT(const size_t &posSection, const size_t &posItem) const
{
    if(m_dvs->sections.size() > posSection)
    {
        if(m_dvs->sections[posSection].items.size() > posItem)
        {
            double quantity = GetItemQuantity(posSection, posItem);
            double priceHT = GetItemPriceHT(GetItemRef(posSection, posItem));

            return (quantity * priceHT);
        }
        else
        {
            throw ticpp::Exception("No item with this pos !");
        }
    }
    else
    {
        throw ticpp::Exception("No section with this pos !");
    }
}

//------------------------------------------------------------------------------
void DevisManager::UpdateItem(const size_t &posSection, const size_t &posItem, const std::string &itemID, const double &quantity)
{
    if(m_dvs->sections.size() > posSection)
    {
        if(m_dvs->sections[posSection].items.size() > posItem)
        {
            if(m_itemDB.find(itemID) != m_itemDB.end())
            {
                m_dvs->sections[posSection].items[posItem].ref = itemID;
                m_dvs->sections[posSection].items[posItem].quantity = quantity;

                BackupSaveCurrent(BACKUP_SECTION_MODIFIED, GetSectionName(posSection));
                m_dvs->isDevisModified = true;
            }
            else
            {
                throw ticpp::Exception("No item with this id !");
            }
        }
        else
        {
            throw ticpp::Exception("No item with this pos !");
        }
    }
    else
    {
        throw ticpp::Exception("No section with this pos !");
    }
}

//------------------------------------------------------------------------------
std::string DevisManager::GetItemRef(const size_t &posSection, const size_t &posItem) const
{
    if(m_dvs->sections.size() > posSection)
    {
        if(m_dvs->sections[posSection].items.size() > posItem)
        {
            return m_dvs->sections[posSection].items[posItem].ref;
        }
        else
        {
            throw ticpp::Exception("No item with this pos !");
        }
    }
    else
    {
        throw ticpp::Exception("No section with this pos !");
    }
}

//------------------------------------------------------------------------------
bool DevisManager::NewItem(const int &posSection)
{
    std::string itemID;
    if(m_itemDB.size() != 0)
    {
        itemID = m_itemDB.rbegin()->first;

        if(m_dvs->sections.size() > posSection)
        {
            StructItem tempItem;
            tempItem.ref = itemID;
            tempItem.quantity = 1;
            m_dvs->sections[posSection].items.push_back(tempItem);
            m_dvs->isDevisModified = true;
            BackupSaveCurrent(BACKUP_SECTION_MODIFIED, GetSectionName(posSection));
            return true;
        }
        else
        {
            throw ticpp::Exception("ItemDB is empty !");
        }
    }
    else
    {
        throw ticpp::Exception("No section with this pos !");
    }
}

//------------------------------------------------------------------------------
bool DevisManager::RemoveItem(const size_t &posSection, const size_t &posItem)
{
    if(m_dvs->sections.size() > posSection)
    {
        if(m_dvs->sections[posSection].items.size() > posItem)
        {
            m_dvs->sections[posSection].items.erase(m_dvs->sections[posSection].items.begin() + posItem);
            m_dvs->isDevisModified = true;
            BackupSaveCurrent(BACKUP_SECTION_MODIFIED, GetSectionName(posSection));
            return true;
        }
        else
        {
            throw ticpp::Exception("No item with this pos !");
        }
    }
    else
    {
        throw ticpp::Exception("No section with this pos !");
    }
}

//------------------------------------------------------------------------------
void DevisManager::LoadFromFile(const std::string &filename)
{
    #if defined USE_MULTIPLE_CONFIG
    SetCurrentDevisId(NewDevis());
    #else
    Close();
    #endif

    m_dvs->isDevisModified = false;

    ticpp::Document myDoc(filename);
    myDoc.LoadFile();

    ticpp::Element *base = myDoc.FirstChild("devismaker_file")->ToElement();
    if(base->FirstChild("version")->ToElement()->GetText(false) == "1")
    {
        // Infos de base :
        ticpp::Element *cfgElement = base->FirstChild("config")->ToElement();
        m_dvs->cfg.LoadFromElement(cfgElement);


        // Sections :
        ticpp::Element *Sections = base->FirstChild("sections")->ToElement();

        ticpp::Element *SectionElement = Sections->FirstChildElement("section", false);
        while(SectionElement != NULL) // Boucle des éléments
        {
            StructSection tempSection;
            tempSection.sectionID = SectionElement->FirstChild("sectionID")->ToElement()->GetText();

            ticpp::Element *Items = SectionElement->FirstChildElement("items");
            ticpp::Element *ItemElement = Items->FirstChildElement("item", false);
            while(ItemElement != NULL)
            {
                StructItem tempItem;
                tempItem.ref = ItemElement->GetAttribute("ref");
                tempItem.quantity = stringToNb<double>(ItemElement->GetAttribute("quantity"));

                if(ItemExists(tempItem.ref))
                {
                    tempSection.items.push_back(tempItem);
                }
                else
                {
                	// /!\ SI l'item n'existe plus, il est supprimé, donc le devis est modifié !
                	m_dvs->isDevisModified = true;
                }
                ItemElement = ItemElement->NextSiblingElement("item", false);
            }

            SectionElement = SectionElement->NextSiblingElement("section", false);

            /*if(SectionExists(tempSection.sectionID)) // Les sessions inexistantes sont autorisés
            {*/
                m_dvs->sections.push_back(tempSection);
            //}
        }

        m_dvs->filename = filename;
        #if not defined USE_MULTIPLE_CONFIG
        m_isOpen = true;
        #endif
        BackupStart(BACKUP_NEW);
    }
    else
    {
        Close();
        throw ticpp::Exception("Bad file version !");
    }
}

//------------------------------------------------------------------------------
std::string DevisManager::GetFilename() const
{
    return m_dvs->filename;
}

//------------------------------------------------------------------------------
bool DevisManager::IsOpen() const
{
    #if defined USE_MULTIPLE_CONFIG
    return !m_dvsMap.empty() && IsOk();
    #else
    return m_isOpen && IsOk();
    #endif
}
//------------------------------------------------------------------------------
bool DevisManager::IsModified() const
{
    if(ItemSectionDBBase::IsModified() == true)
    {
        throw ticpp::Exception("Error : ItemSectionDBBase is modified in DevisManager !");
    }

    return(IsOpen() && (m_dvs->isDevisModified || m_dvs->cfg.IsModified()));
}
//------------------------------------------------------------------------------
// Backup
bool DevisManager::BackupCanGoTo(const int &pos) const
{
    // Si il y a au moins un backup et que la position Test appartient au backup.
    int testPos = m_dvs->backupPos + pos;
    return(m_dvs->backup.size() > 1 && testPos >= 0 && testPos <= m_dvs->backup.size() - 1);
}
//------------------------------------------------------------------------------
void DevisManager::BackupSaveCurrent(const int &type, const std::string &infoStr)
{
    if(m_dvs->backup.size() > 1 && m_dvs->backupPos != m_dvs->backup.size() - 1) // Si il y a des backup après la pos actuelle
    {
        // ==> Il faut les supprimer (on commence une nouvelle version)
        m_dvs->backup.erase(m_dvs->backup.begin() + m_dvs->backupPos + 1,  m_dvs->backup.end());
    }

    m_dvs->backup.resize(m_dvs->backup.size() + 1); // Ajout d'un enregistrement.

    // Data :
    int pos = m_dvs->backup.size() - 1;
    m_dvs->backup[pos].sections = m_dvs->sections;
    m_dvs->backup[pos].isDevisModified = m_dvs->isDevisModified;

    m_dvs->backup[pos].type = type;
    m_dvs->backup[pos].infoStr = infoStr;

    // Epuration...
    if(m_dvs->backup.size() > BACKUP_NB) // Si trop de backup...
    {
        m_dvs->backup.erase(m_dvs->backup.begin()); // On supprime le premier (le plus ancien).
    }

    // Current backup ==> Le dernier.
    m_dvs->backupPos = m_dvs->backup.size() - 1;
}
//------------------------------------------------------------------------------
void DevisManager::BackupLoad(const int &pos)
{
    int newPos = m_dvs->backupPos + pos;
    if(newPos >= 0 && newPos < m_dvs->backup.size()) // Si nouvelle pos dans le tableau.
    {
        m_dvs->backupPos = newPos;
		m_dvs->sections = m_dvs->backup[m_dvs->backupPos].sections;
		m_dvs->isDevisModified = m_dvs->backup[m_dvs->backupPos].isDevisModified;
    }
    else
    {
        throw ticpp::Exception("Backup out of range !");
    }
}
//------------------------------------------------------------------------------
int DevisManager::BackupGetType(const int &pos) const
{
    int newPos = m_dvs->backupPos + pos;
    if(newPos >= 0 && newPos < m_dvs->backup.size()) // Si nouvelle pos dans le tableau.
    {
        return m_dvs->backup[newPos].type;
    }
    else
    {
        throw ticpp::Exception("Backup out of range !");
    }
}
//------------------------------------------------------------------------------
std::string DevisManager::BackupGetInfoStr(const int &pos) const
{
    int newPos = m_dvs->backupPos + pos;
    if(newPos >= 0 && newPos < m_dvs->backup.size()) // Si nouvelle pos dans le tableau.
    {
        return m_dvs->backup[newPos].infoStr;
    }
    else
    {
        throw ticpp::Exception("Backup out of range !");
    }
}
//------------------------------------------------------------------------------
void DevisManager::BackupClear()
{
    m_dvs->backup.clear();
    m_dvs->backupPos = 0;
}
//------------------------------------------------------------------------------
void DevisManager::BackupStart(const int &type, const std::string &infoStr)
{
    BackupClear();
    BackupSaveCurrent(type, infoStr);
}
//------------------------------------------------------------------------------
int DevisManager::GetSectionCount() const
{
    return m_dvs->sections.size();
}
//------------------------------------------------------------------------------
int DevisManager::GetItemCountInAllSections() const
{
    int total = 0;

    for(int i = 0; i < m_dvs->sections.size(); i++)
    {
        total += m_dvs->sections[i].items.size();
    }

    return total;
}
//------------------------------------------------------------------------------
// Gestion de plusieurs devis
#if defined USE_MULTIPLE_CONFIG
/*int DevisManager::GetCurrentDevisPos() const
{
    return m_currentDevisPos;
}*/
//------------------------------------------------------------------------------
/*int DevisManager::GetDevisCount() const
{
    return m_dvsMap.size();
}*/
//------------------------------------------------------------------------------

void DevisManager::SetCurrentDevisId(const int &id)
{
    if(m_dvsMap.find(id) != m_dvsMap.end())
    {
        m_currentDevisId = id;
        m_dvs = &m_dvsMap[m_currentDevisId];
    }
    else
    {
        std::string msg = "SetCurrentDevisId : id not found !\n";
        msg += nbToString(id);
        throw ticpp::Exception(msg.c_str());
    }
}
//------------------------------------------------------------------------------
int DevisManager::GetCurrentDevisId() const
{
    return m_currentDevisId;
}
//------------------------------------------------------------------------------
/*void DevisManager::SetCurrentDevisPos(const int &pos)
{
    if(pos >= 0 && pos < m_dvsMap.size())
    {
        m_currentDevisPos = pos;
        m_dvs = &m_dvsMap[pos];
    }
    else
    {
        std::string msg = "SetCurrentDevisPos : pos negative / too big !\n";
        msg += nbToString(pos);
        throw ticpp::Exception(msg.c_str());
    }
}*/
//------------------------------------------------------------------------------
void DevisManager::RemoveDevis(const int &id)
{
    if(m_dvsMap.find(id) != m_dvsMap.end())
    {
        m_dvsMap.erase(m_dvsMap.find(id));
    }
    else
    {
        std::string msg = "SetCurrentDevisPos : id negative / too big !\n";
        msg += nbToString(id);
        throw ticpp::Exception(msg.c_str());
    }
}
//------------------------------------------------------------------------------
int DevisManager::CopyDevis(const int &id)
{
    if(m_dvsMap.find(id) != m_dvsMap.end())
    {
        m_LastDevisId++;
        m_dvsMap[m_LastDevisId] = m_dvsMap[id];
        return m_LastDevisId;
    }
    else
    {
        std::string msg = "SetCurrentDevisPos : id negative / too big !\n";
        msg += nbToString(id);
        throw ticpp::Exception(msg.c_str());
    }
}
//------------------------------------------------------------------------------
int DevisManager::NewDevis()
{
    StructDevis tempDevis;
    tempDevis.cfg.ClearAll();
    tempDevis.sections.clear();
    tempDevis.filename = "";
    tempDevis.isDevisModified = false;

    m_LastDevisId++;
    m_dvsMap[m_LastDevisId] = tempDevis;

    return m_LastDevisId;
}
//------------------------------------------------------------------------------
int DevisManager::GetLastCreatedDevisId() const
{
    return m_LastDevisId;
}
#endif
